function love.load()
	require("conf.lua")
	require("keybinds.lua")

	require("json/json") --awww yeah, licensed modules :D

	function requiredir(dir) --spacesaver 9000

		for k, v in pairs(love.filesystem.enumerate(dir)) do
			require(dir.."/"..v)
		end

	end

	requiredir("/ext")
	requiredir("/ents")
	require("mods/loader.lua")
	
	gw = 60
	gh = 60
	vw = 40
	vh = 40
	gz = 40
	pw = 10
	ph = 10
	o, h = vw * pw, vh * ph
	pixels = {}
	df = {}
	dl = {}
	ents = {}
	items = {}
	levels = {}
	level = ""
	usemenu = true
	menu = {}
	pause = false
	lfont = love.graphics.newFont(18)
	sfont = love.graphics.newFont(12)
	message = ""
	view = View()
	if not usemenu then
		love.graphics.setMode(vw * pw, vh * ph)
	else
		love.graphics.setMode((vw * pw) * 1.5, vh * ph)
	end
	love.filesystem.setIdentity("Grid")
	if not love.filesystem.exists("saves") then love.filesystem.mkdir("saves") end
	Main()
	hook.Call("Init")
end

local nm = 0

function Pause(t)

	pause = t

end

function love.update(dt)
	checktimers()
	if nm < os.clock() and not pause then
		--Tick
		
		for k, v in pairs(ents) do
		
			if v.Tick then v:Tick() end
		
		end
		
		hook.Call("Tick")
		
		nm = os.clock() + .2
	end
	hook.Call("Update", dt)
end

function SetColor(c)

	love.graphics.setColor(c.r, c.g, c.b, c.a)

end

function DrawRect(x, y, w, h, c)

	SetColor(c)
	love.graphics.rectangle("fill", x, y, w, h)

end

function DrawLight(x, y, c)

	dl[x] = dl[x] or {}
	dl[x][y] = dl[x][y] or Color(0, 0, 0, 0)
	dl[x][y] = dl[x][y] + c

end

function RegisterEnt(e, c)

	e.class = c
	e.__entindex = #ents + 1
	ents[#ents + 1] = e

end

function love.draw()
	love.graphics.clear()
	for k, v in pairs(ents) do
	
		if v.Draw then v:Draw() end
	
	end
	for x, t in pairs(dl) do
	
		for y, c in pairs(t) do
		
			SetPixel(x, y, c)
			
		end
	
	end
	dl = {}
	view:Draw(function() 
		for k, v in pairs(pixels) do
		
			local x, y, c = v.x, v.y, v.c
			DrawRect((x * pw) - pw, (y * ph) - ph, pw, ph, c)
		
		end
	end)
	pixels = {}
	local g = love.graphics
	--Start Menu Stuff Here
	
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.line(o, 0, o, h)
	
	if usemenu and menu then
		menu:Draw()
	end
	
	love.graphics.print(message, o + 10, h - 50)
	love.graphics.print("version 0.04", o + 10, h - 20)
	if pause then love.graphics.print("PAUSED", love.graphics.getWidth() - 50, h - 20) end
	
	--End Menu Stuff]]
	g.setColor(255, 255, 255, 255)
	g.print("FPS: "..math.floor(1/love.timer.getDelta()), 4, 0)
	hook.Call("Draw")
end

function RegLev(n, f)

	levels[n] = f

end

function LoadLevel(n)

	require("levels/"..n)
	ents = {}
	levels[n]() --kill kill kill
	level = n

end

function SaveFiles()

	return love.filesystem.enumerate("saves")

end

function SaveGame(n)

	s = {ents, level}
	love.filesystem.write("saves/"..n..".txt", json.encode(s), all)
	UpdateMenuSet(menus)
	hook.Call("Save", n)

end

function LoadGame(n)

	local f = json.decode(love.filesystem.read("saves/"..n, all))
	LoadLevel(f[2])
	ents = {}
	for k, v in pairs(f[1]) do
	
		ents[k] = v
		--file.Write("out.txt", json.encode(v))
	
	end
	hook.Call("Load", n)

end

function Main() --main stuff, I guess

	menus = {}
	menus.root = Menu("Main Menu")
	menus.root:Add("Load Level", function() SetMenu(menus.levels) end)
	menus.root:Add("Save Game", function() SetMenu(menus.save) end)
	menus.root:Add("Load Game", function() SetMenu(menus.load) end)
	menus.levels = Menu("Load Level")
	for k, v in pairs(love.filesystem.enumerate("levels")) do
		menus.levels:Add(v, function() LoadLevel(v) end)
	end
	menus.levels:Add("Back", function() SetMenu(menus.root) end)
	menus.save = Menu("Save Game")
	for k, v in pairs(SaveFiles()) do
		menus.save:Add(v, function() SaveGame(v) end)
	end
	menus.save:Add("[New Save]", function() SaveGame("Save"..table.size(SaveFiles()) + 1) end)
	menus.save:Add("Back", function() SetMenu(menus.root) end)
	menus.load = Menu("Load Game")
	for k, v in pairs(SaveFiles()) do
		menus.load:Add(v, function() LoadGame(v) end)
	end
	menus.load:Add("Back", function() SetMenu(menus.root) end)
	SetMenu(menus.root)
	LoadLevel("test.lua")

end